/*
 * World.java
 *
 * Created on Streda, 2007, november 21, 19:21
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package cz.muni.fi.pb162.traffic.common;

import cz.muni.fi.pb162.traffic.network.Line;
import cz.muni.fi.pb162.traffic.network.Network;
import cz.muni.fi.pb162.traffic.network.Stop;
import cz.muni.fi.pb162.traffic.tickets.Tickets;
import cz.muni.fi.pb162.traffic.turnstiles.Dispatcher;
import cz.muni.fi.pb162.traffic.turnstiles.Turnstiles;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 *
 * @author misko
 */
public class World {

    private Network network = Network.getInstance();
    private List<Stop> stops = new ArrayList<Stop>();
    private Tickets tickets = Tickets.getInstance();
    private Turnstiles turnstiles = Turnstiles.getInstance();

    /** Creates a new instance of World */
    public void createWorld() throws FileNotFoundException, IOException {
        createBrnoNetwork();
        createTurnstiles();
        createTickets();
    }

    @SuppressWarnings("unchecked")
    private void createBrnoNetwork() throws FileNotFoundException, IOException {
        Set<Stop> stopsSet = new LinkedHashSet<Stop>();
        BufferedReader reader = new BufferedReader(new InputStreamReader(
                new FileInputStream("velke brno.txt"), "UTF-8"));
        Line line = null;
        while (reader.ready()) {

            String record = reader.readLine();
//            System.out.println(record);
            if (record.startsWith("#")) {
                line = new Line(record.substring(1), false);
                network.addLine(line);
                continue;
            }
            if (record.startsWith("@")) {
                line = new Line(record.substring(1), true);
                network.addLine(line);
                continue;
            }
            final Stop stop = new Stop(record);
            line.addStop(stop);
            stopsSet.add(stop);
        }
        stops.addAll(stopsSet);
//        @SuppressWarnings("unchecked")
//        List<Stop> sortedStops = new ArrayList<Stop>();
//        sortedStops.addAll(stops);
//        Collections.sort(stops, new Comparator() {
//
//            public int compare(Object o1, Object o2) {
//                return ((Stop) o1).getName().compareTo(((Stop) o2).getName());
//            }
//        });
//        System.out.println(sortedStops);

    }

    public List<Stop> getStops() {
        return Collections.unmodifiableList(stops);
    }


    private void createNetwork() {


        Stop a = new Stop("a");
        Stop b = new Stop("b");
        Stop c = new Stop("c");
        Stop d = new Stop("d");
        Stop e = new Stop("e");
        Stop f = new Stop("f");

        Stop k = new Stop("k");
        Stop l = new Stop("l");
        Stop m = new Stop("m");
        Stop n = new Stop("n");
        Stop o = new Stop("o");

        Stop p = new Stop("p");
        Stop q = new Stop("q");
        Stop r = new Stop("r");
        Stop s = new Stop("s");
        Stop t = new Stop("t");
        Stop u = new Stop("u");
        Stop v = new Stop("v");

        Stop x = new Stop("x");
        Stop y = new Stop("y");
        Stop z = new Stop("z");


        Line lineA = new Line("A", false);
        Line lineB = new Line("B", false);
        Line lineC = new Line("C", false);
        Line lineD = new Line("D", true);

        lineA.addStop(a);
        lineA.addStop(b);
        lineA.addStop(c);
        lineA.addStop(d);
        lineA.addStop(e);
        lineA.addStop(f);

        lineB.addStop(k);
        lineB.addStop(l);
        lineB.addStop(m);
        lineB.addStop(c);
        lineB.addStop(n);
        lineB.addStop(o);

        lineC.addStop(c);
        lineC.addStop(x);
        lineC.addStop(y);
        lineC.addStop(z);

        lineD.addStop(p);
        lineD.addStop(q);
        lineD.addStop(l);
        lineD.addStop(r);
        lineD.addStop(y);
        lineD.addStop(s);
        lineD.addStop(t);
        lineD.addStop(e);
        lineD.addStop(u);
        lineD.addStop(n);
        lineD.addStop(v);
        lineD.addStop(b);

        network.addLine(lineA);
        network.addLine(lineB);
        network.addLine(lineC);
        network.addLine(lineD);
    }

    private void createTickets() {
        tickets.newTicket(14);
        tickets.newTicket(5);
        tickets.newTicket(300);
    }

    private void createTurnstiles() {
        turnstiles.addOrMoveTurnstile("0041", new Stop("Česká"));
        turnstiles.addOrMoveTurnstile("0101", new Stop("Grohova"));
        turnstiles.addOrMoveTurnstile("1231", new Stop("Klusáčkova"));
        turnstiles.addOrMoveTurnstile("a78f", new Stop("Botanická"));
        turnstiles.addOrMoveTurnstile("1747", new Stop("Hrnčířská"));

        turnstiles.addOrMoveTurnstile("7z7z", new Stop("Rybkova"));
        turnstiles.addOrMoveTurnstile("6828", new Stop("Česká"));
        turnstiles.addOrMoveTurnstile("1314", new Stop("Hlavní nádraží"));
        turnstiles.addOrMoveTurnstile("54kc", new Stop("Náměstí Svobody"));
    }

    public void destroyWorld() {
        network.reset();
        tickets.reset();
        turnstiles.reset();
    }

    public Network getNetwork() {
        return network;
    }

    public Dispatcher getDispatcher() {
        return Dispatcher.getInstance();
    }

    public Tickets getTickets() {
        return tickets;
    }

    public Turnstiles getTurnstiles() {
        return turnstiles;
    }
}
