/*
 * NetworkTest.java
 * JUnit based test
 *
 * Created on Pondelok, 2007, november 19, 13:49
 */
package cz.muni.fi.pb162.traffic.network;

import cz.muni.fi.pb162.traffic.common.World;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;
import junit.framework.TestCase;

/**
 *
 * @author misko
 */
public class NetworkTest extends TestCase {

    private World world = new World();
    private Stop ceska = new Stop("Česká");
    private Stop moravak = new Stop("Moravské náměstí");
    private Stop grohova = new Stop("Grohova");
    private Stop komenskeho = new Stop("Komenského náměstí");
    private Stop svobodak = new Stop("Náměstí Svobody");
    private Stop divadlo = new Stop("Janáčkovo divadlo");
    private Stop autobusak = new Stop("Autobusové nádraží");
    private Stop zvonarka = new Stop("Zvonařka");
    private Stop mirak = new Stop("NÁMĚSTÍ MÍRU");
    private Stop heinrichova = new Stop("Heinrichova");
    private Stop smetanova = new Stop("Smetanova");
    private static int ITERATIONS = 10000;

    public NetworkTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        world.createWorld();
    }

    protected void tearDown() throws Exception {
        world.destroyWorld();
    }

    public void testNextStopsToMirak() {
        Set<Stop> nextMirak = new HashSet<Stop>();
        nextMirak.add(heinrichova);

        assertEquals(nextMirak, world.getNetwork().getNextStops(mirak));

    }

    public void testNextStopsToAutobusoveNadrazi() {
        Set<Stop> nextAutobusak = new HashSet<Stop>();
        nextAutobusak.add(zvonarka);


        assertEquals(nextAutobusak, world.getNetwork().getNextStops(autobusak));

    }

    /**
     * Test of distance method, of class traffic.Network.
     */
    public void testNextStopsToCeska() {
        Set<Stop> nextCeska = new HashSet<Stop>();
        nextCeska.add(divadlo);
        nextCeska.add(grohova);
        nextCeska.add(komenskeho);
        nextCeska.add(moravak);
        nextCeska.add(svobodak);
        nextCeska.add(smetanova);


        assertEquals(nextCeska, world.getNetwork().getNextStops(ceska));

    }

    /**
     * Test of distance method, of class traffic.Network.
     */
    public void testDistanceBotanicka2Ceska() {
        assertEquals(4, Network.getInstance().distance(new Stop("Botanická"), new Stop("Česká")));
    }
    public void testDistanceBotanicka2Grohova() {
        assertEquals(5, Network.getInstance().distance(new Stop("Botanická"), new Stop("Grohova")));
    }

    public void testDistanceHrncirska2Botanicka() {
        assertEquals(7, Network.getInstance().distance(new Stop("Hrnčířská"), new Stop("Botanická")));
    }
    public void testDistanceMalomMost2Hlavak() {
        assertEquals(9, Network.getInstance().distance(new Stop("MALOMĚŘICKÝ MOST"), new Stop("Hlavní nádraží")));
    }

    public void testDistanceBranka2Hlavak() {
        assertEquals(12, Network.getInstance().distance(new Stop("Branka"), new Stop("Hlavní nádraží")));


    }

    public void testDistanceMalomMost2Branka() {
        assertEquals(15, Network.getInstance().distance(new Stop("MALOMĚŘICKÝ MOST"), new Stop("Branka")));
    }

    /**
     * Test time of distance method, of class traffic.Network.
     */
    public void testDistanceTime() {
        Random random = new Random();

        List<Stop> stops = world.getStops();
        int stopsNo = stops.size();
        long time = 0;
        for (int i = 0; i < ITERATIONS; i++) {
            Stop start = stops.get(random.nextInt(stopsNo));
            Stop end = stops.get(random.nextInt(stopsNo));

            long startTime = System.currentTimeMillis();
            world.getNetwork().distance(start, end);
            time += System.currentTimeMillis() - startTime;
        }

        double oneIteration = time * 100 / ITERATIONS;
        System.out.println(ITERATIONS + " iterations take " + time + " ms");
        System.out.println("one path finding takes approximately " + oneIteration + " µs");
    }

    /**
     * Test time of distance method, of class traffic.Network.
     */
//    public void testFastDistance() {
//        Random random = new Random();
//        List<Stop> stops = world.getStops();
//        int stopsNo = stops.size();
//
//        for (int i = 0; i < 2000; i++) {
//            Stop start = stops.get(random.nextInt(stopsNo));
//            Stop end = stops.get(random.nextInt(stopsNo));
////            System.out.println(start+"–"+end);
//            assertEquals(
//                    world.getNetwork().distance(start, end),
//                    world.getNetwork().fastDistance(start, end));
////                    world.getNetwork().distance(start,end));
//        }
//
//    }

    /**
     * Test time of fastDistance method, of class traffic.Network.
     */
//    public void testFastDistanceTime() {
//        Random random = new Random();
//
//        List<Stop> stops = world.getStops();
//        int stopsNo = stops.size();
//        long time = 0;
//        for (int i = 0; i < ITERATIONS; i++) {
//            Stop start = stops.get(random.nextInt(stopsNo));
//            Stop end = stops.get(random.nextInt(stopsNo));
//
//            long startTime = System.currentTimeMillis();
//            world.getNetwork().fastDistance(start, end);
//            time += System.currentTimeMillis() - startTime;
//        }
//
//        double oneIteration = time * 100 / ITERATIONS;
//        System.out.println(ITERATIONS + " iterations take " + time + " ms");
//        System.out.println("one fast path finding takes approximately " + oneIteration + " µs");
//    }
}
